/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    alignMeshSurf

Description
  	- Moves mesh points in the Z direction so that the bottom and top
	  faces are aligned with provided surfaces,
	- moves all internal mesh points,
	- and writes the new mesh to next time.

\*---------------------------------------------------------------------------*/


#include "argList.H"
#include "Time.H"
#include "fvCFD.H"
#include "polyMesh.H"

#include "triSurface.H"
#include "triSurfaceTools.H"
#include "triSurfaceSearch.H"
#include "octreeDataTriSurface.H"
#include "octree.H"
#include "pointIndexHit.H"
#include "OFstream.H"

using namespace Foam;
using namespace Foam::triSurfaceTools;

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

	argList::noParallel();
	argList::validArgs.append("bottom surface file");
	argList::validArgs.append("top surface file");
	

#   include "setRootCase.H"
#   include "createTime.H"
#   include "createMesh.H"

	// read the surfaces
	Info << "Reading surfaces ..." << nl << nl 
		 << "BOTTOM :" << nl << endl;
	
	triSurface surfB(args.args()[3]);
	surfB.writeStats(Pout);
	
	Info << nl << "TOP :" << nl << endl;
		
	triSurface surfT(args.args()[4]);
	surfT.writeStats(Pout);
	
	// min Z of bottom surface bounding box
	scalar surfMinZ = boundBox(surfB.points()).min().component(2);
	
	// max Z of top surface bounding box
	scalar surfMaxZ = boundBox(surfT.points()).max().component(2);
	
	// Search engines on surfaces
	triSurfaceSearch querySurfB(surfB);
	const octree<octreeDataTriSurface>& treeB = querySurfB.tree();
	
	triSurfaceSearch querySurfT(surfT);
	const octree<octreeDataTriSurface>& treeT = querySurfT.tree();
	
	// old and new mesh point fields
	const pointField& pts = mesh.points();
	pointField newPts(mesh.points());
	
	// min and max Z of mesh bounding box
	scalar bottomMesh = boundBox(pts).min().component(2);
	scalar topMesh = boundBox(pts).max().component(2);
	
	Info << nl << "Original mesh bounding box : "
		 << boundBox(pts) << nl << endl;

	forAll(pts, pointI)
	{
		const point& pt = pts[pointI];
		
		scalar ptX = pt.component(0);
		scalar ptY = pt.component(1);
		scalar ptZ = pt.component(2);
		
		// point below and above the surface
		point plow (ptX,ptY,surfMinZ-1);
		point phigh (ptX,ptY,surfMaxZ+1);
		
		// get the bottom surface height
		scalar bottomSurf = treeB.findLine(plow, phigh).hitPoint().component(2);
		
		// get the top surface height
		scalar topSurf = treeT.findLine(plow, phigh).hitPoint().component(2);
		
		// relative Z position of the mesh point
		// it is based on original mesh bounding box and NOT local bottom
		// and top mesh faces
		scalar meshZRatio = (ptZ-bottomMesh) / (topMesh-bottomMesh);
		
		// new point height
		newPts[pointI].component(2) = bottomSurf + meshZRatio * (topSurf-bottomSurf);
	}
	
	Info << nl << "Moving mesh points... " << nl << endl;
		
	// move mesh point to new locations
	mesh.polyMesh::movePoints(newPts);
	
	Info << nl << "NEW mesh bounding box : "
		 << boundBox(mesh.points()) << nl << endl;
	
	// write mesh points to new time step directory
	runTime++;
	mesh.write();
	
	Info << nl << "End " << nl << endl;
	
	return(0);
}

// ************************************************************************* //
